@@ -471,7 +471,7 @@ struct SizeSpec {
   }
 };
 
-#ifdef XP_UNIX
+#if XP_UNIX | HAVE_XRM
 #include <X11/Xlib.h>
 #include <X11/Xresource.h>
 #include <X11/Xutil.h>
@@ -730,8 +730,9 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
   //
   // This is an attempt to fix bug 20573.
   //
-#ifdef XP_UNIX
-  AppendXRDBGeometryFeatures( aName, featuresSpecified, &features);
+#if XP_UNIX | HAVE_XRM
+  if(!aDialog)
+    AppendXRDBGeometryFeatures( aName, featuresSpecified, &features);
 #endif
 
   // Make sure we call CalculateChromeFlags() *before* we push the
